@@ -15,33 +15,39 @@ Usage:
     {} <remote_peer>
 """.format(__file__)
 
-from ike.protocol import IKE, parse_packet, State
-
 
 class IKEInitiator(asyncio.DatagramProtocol):
-    def __init__(self):
-        self.ike = IKE()
-
+    """
+    Implements an IKE initiator that attempt to negotiate a single child SA
+    to remote peer.
+    """
     def connection_made(self, transport):
+        from ike.protocol import IKE
         self.transport = transport
+        sock = self.transport.get_extra_info("socket")
+        address = sock.getsockname()
+        peer = sock.getpeername()
+        logger.debug("Socket created from {} to {}".format(address, peer))
+        self.ike = IKE(address, peer)
         logger.info("Sending INIT")
-        self.transport.sendto(self.ike.init())  # no need for address
+        self.transport.sendto(self.ike.init_send())  # no need for address
 
     def datagram_received(self, data, address):
+        from ike.protocol import State
+        from ike.const import ExchangeType
         (host, port) = address
         logger.info("Received %r from %s:%d" % (data, host, port))
-        # TODO: Read SPIs and Exchange type and decide what to do based on that instead of
-        # doing it like this:
-        packet = parse_packet(data=data, ike=self.ike)
-        logger.debug("Got responder SPI: {0:x}".format(packet.rSPI))
-        self.ike.rSPI = packet.rSPI
-        self.ike.packets.append(packet)
-        if self.ike.state == State.INIT:
-            ike_auth = self.ike.auth()
+        packet = self.ike.parse_packet(data=data)
+        logger.info('{} message {}'.format(packet.exchange_type.name, packet.message_id))
+        # TODO: Check that SPIs match
+        if self.ike.state == State.INIT and packet.exchange_type == ExchangeType.IKE_SA_INIT:
+            self.ike.init_recv()
+            ike_auth = self.ike.auth_send()
             logger.info("Sending AUTH")
             self.transport.sendto(ike_auth)
-            # self.ike.packets.append(Packet(data=ike_auth))
             logger.info("IKE AUTH SENT")
+        elif self.ike.state == State.AUTH and packet.exchange_type == ExchangeType.IKE_AUTH:
+            self.ike.auth_recv()
 
     # Possibly invoked if there is no server listening on the
     # address to which we are sending.
@@ -60,8 +66,7 @@ def main(peer):
 
 if __name__ == '__main__':
     opts = docopt.docopt(__doc__)
-    logging.basicConfig(level=logging.DEBUG)
+    logging.basicConfig(level=logging.DEBUG, format='%(levelname)s:%(name)s.%(funcName)s:[%(lineno)s]: %(message)s')
     logger = logging.getLogger('MAIN')
-    logger.setLevel(logging.DEBUG)
     logger.info("Starting...")
     main(opts['<remote_peer>'])
